from io import FileIO
from django.db.models.fields.files import FieldFile
from magic import from_buffer

def meme_type(file:FileIO|FieldFile) -> tuple[str, str]:
    """detect meme type of file by reading the first `2048 bytes` of file

    Parameters
    ----------
    file : FileIO
        file you want to detect meme type of it

    Returns
    -------
    tuple[str, str]
        type and subtype of file for example `("Image","png")`
    """

    typ, subtyp = from_buffer(file.read(2048), True).split('/')
    return typ, subtyp



from io import FileIO

from django.core.exceptions import ValidationError
from django.utils.deconstruct import deconstructible

from django_ffield.utils import meme_type

@deconstructible
class FileTypeValidator:
    """A file type validator
    """

    def __init__(self, allowd_types: list[str],disallowed_types=[]) -> None:
        """
        Parameters
        ----------
        allowed_types : list[str]
            types and sub-types that allowd file be
            example :
                `["image","pdf","mp4",...]`
        
        disallowed_types : list[str]
            types and sub-types that disallowed file be
            example :
                `["py","exe","bat",...]`
        
        """
        self.allowd_types = list(map(str.lower, set(allowd_types)))
        self.disallowed_types = list(map(str.lower, set(disallowed_types)))

    def __call__(self, file:FileIO) -> None:
        """called by django validator

        Parameters
        ----------
        file : FileIO
            the file input that passed by django to validate 

        Raises
        ------
        ValidationError
            when the file don't match by selected types this error raises to show to user
        """
        typ, formt = meme_type(file)
        
        # * to prevent forbidden upload files by user
        if self.allowd_types  and  typ not in self.allowd_types and formt not in self.allowd_types:
            raise ValidationError(
                f"Unsupported Format: The {formt} type is not in allowd supported formats. Supported formats: {self.allowd_types}. Unsupported formats: {self.disallowed_types}. "
            )
        elif self.disallowed_types  and typ in self.disallowed_types or formt in self.disallowed_types :
            raise ValidationError(
                f"Unsupported Format: The {formt} type is not supported formats. Unsupported formats: {self.disallowed_types}."
            )


from typing import Any
from django.db.models.base import Model
from django.db.models.fields.files import FieldFile, FileField
from django_ffield.validators import FileTypeValidator
from django_ffield.utils import meme_type


class FiledFField(FieldFile):
    def __init__(self, instance: Model, field: FileField, name: str | None) -> None:
        super().__init__(instance, field, name)

    @property
    def mime_type(self) -> str:
        """return the mimetype of file

        Returns
        -------
        str
            mimetype : `'image/png'`
        """
        mime: str = '/'.join(meme_type(self))
        return mime

    @property
    def type(self) -> str:
        """return the type of file for example : `Video,Image,...`

        Returns
        -------
        str
            type of file
        """
        return meme_type(self)[0]

    @property
    def format(self) -> str:
        """return subtype of file or extention like `png,mp4,pdf,...`

        Returns
        -------
        str
            the subtype of file
        """
        return meme_type(self)[1]


class FileFField(FileField):
    """This field can accept a list of `allowd_types` as and `disallowd_types` input,
    allowing the user to pass only these specific allowd_types to the field,
    and don't allowd pass disallowd types to the filed.

    Parameters
    ----------
    FileField : Django FileField

    """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        self.allowd_types: list[str] = kwargs.pop("allowd_types", [])
        self.disallowd_types: list[str] = kwargs.pop("disallowd_types", [])
        super().__init__(*args, **kwargs)

        if self.allowd_types or self.disallowd_types:
            self.validators.append(
                FileTypeValidator(allowd_types=self.allowd_types,
                                  disallowd_types=self.disallowd_types,)
            )

    attr_class = FiledFField
